@@ -12,8 +12,7 @@
 #include "html.h"
 
 static const char cgit_doctype[] =
-"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"
-"  \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
+"<!DOCTYPE html>\n";
 
 static char *http_date(time_t t)
 {
@@ -607,35 +606,23 @@ void cgit_submodule_link(const char *class, char *path, const char *rev)
 		path[len - 1] = tail;
 }
 
-static const char *fmt_date(time_t secs, const char *format, int local_time)
+const struct date_mode *cgit_date_mode(enum date_mode_type type)
 {
-	static char buf[64];
-	struct tm *time;
-
-	if (!secs)
-		return "";
-	if (local_time)
-		time = localtime(&secs);
-	else
-		time = gmtime(&secs);
-	strftime(buf, sizeof(buf)-1, format, time);
-	return buf;
+	static struct date_mode mode;
+	mode.type = type;
+	mode.local = ctx.cfg.local_time;
+	return &mode;
 }
 
-void cgit_print_date(time_t secs, const char *format, int local_time)
-{
-	html_txt(fmt_date(secs, format, local_time));
-}
-
-static void print_rel_date(time_t t, double value,
+static void print_rel_date(time_t t, int tz, double value,
 	const char *class, const char *suffix)
 {
 	htmlf("<span class='%s' title='", class);
-	html_attr(fmt_date(t, FMT_LONGDATE, ctx.cfg.local_time));
+	html_attr(show_date(t, tz, cgit_date_mode(DATE_ISO8601)));
 	htmlf("'>%.0f %s</span>", value, suffix);
 }
 
-void cgit_print_age(time_t t, time_t max_relative, const char *format)
+void cgit_print_age(time_t t, int tz, time_t max_relative)
 {
 	time_t now, secs;
 
@@ -648,34 +635,34 @@ void cgit_print_age(time_t t, time_t max_relative, const char *format)
 
 	if (secs > max_relative && max_relative >= 0) {
 		html("<span title='");
-		html_attr(fmt_date(t, FMT_LONGDATE, ctx.cfg.local_time));
+		html_attr(show_date(t, tz, cgit_date_mode(DATE_ISO8601)));
 		html("'>");
-		cgit_print_date(t, format, ctx.cfg.local_time);
+		html_txt(show_date(t, tz, cgit_date_mode(DATE_SHORT)));
 		html("</span>");
 		return;
 	}
 
 	if (secs < TM_HOUR * 2) {
-		print_rel_date(t, secs * 1.0 / TM_MIN, "age-mins", "min.");
+		print_rel_date(t, tz, secs * 1.0 / TM_MIN, "age-mins", "min.");
 		return;
 	}
 	if (secs < TM_DAY * 2) {
-		print_rel_date(t, secs * 1.0 / TM_HOUR, "age-hours", "hours");
+		print_rel_date(t, tz, secs * 1.0 / TM_HOUR, "age-hours", "hours");
 		return;
 	}
 	if (secs < TM_WEEK * 2) {
-		print_rel_date(t, secs * 1.0 / TM_DAY, "age-days", "days");
+		print_rel_date(t, tz, secs * 1.0 / TM_DAY, "age-days", "days");
 		return;
 	}
 	if (secs < TM_MONTH * 2) {
-		print_rel_date(t, secs * 1.0 / TM_WEEK, "age-weeks", "weeks");
+		print_rel_date(t, tz, secs * 1.0 / TM_WEEK, "age-weeks", "weeks");
 		return;
 	}
 	if (secs < TM_YEAR * 2) {
-		print_rel_date(t, secs * 1.0 / TM_MONTH, "age-months", "months");
+		print_rel_date(t, tz, secs * 1.0 / TM_MONTH, "age-months", "months");
 		return;
 	}
-	print_rel_date(t, secs * 1.0 / TM_YEAR, "age-years", "years");
+	print_rel_date(t, tz, secs * 1.0 / TM_YEAR, "age-years", "years");
 }
 
 void cgit_print_http_headers(void)
@@ -714,7 +701,6 @@ void cgit_redirect(const char *url, bool permanent)
 	html("Location: ");
 	html_url_path(url);
 	html("\n\n");
-	exit(0);
 }
 
 static void print_rel_vcs_link(const char *url)
@@ -736,7 +722,7 @@ void cgit_print_docstart(void)
 
 	char *host = cgit_hosturl();
 	html(cgit_doctype);
-	html("<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='en' lang='en'>\n");
+	html("<html lang='en'>\n");
 	html("<head>\n");
 	html("<title>");
 	html_txt(ctx.page.title);
@@ -792,7 +778,7 @@ void cgit_print_docend(void)
 	else {
 		htmlf("<div class='footer'>generated by <a href='http://git.zx2c4.com/cgit/about/'>cgit %s</a> at ",
 			cgit_version);
-		cgit_print_date(time(NULL), FMT_LONGDATE, ctx.cfg.local_time);
+		html_txt(show_date(time(NULL), 0, cgit_date_mode(DATE_ISO8601)));
 		html("</div>\n");
 	}
 	html("</div> <!-- id=cgit -->\n");
@@ -805,13 +791,11 @@ void cgit_print_error_page(int code, const char *msg, const char *fmt, ...)
 	ctx.page.expires = ctx.cfg.cache_dynamic_ttl;
 	ctx.page.status = code;
 	ctx.page.statusmsg = msg;
-	cgit_print_http_headers();
-	cgit_print_docstart();
-	cgit_print_pageheader();
+	cgit_print_layout_start();
 	va_start(ap, fmt);
 	cgit_vprint_error(fmt, ap);
 	va_end(ap);
-	cgit_print_docend();
+	cgit_print_layout_end();
 }
 
 void cgit_print_layout_start(void)
@@ -953,14 +937,14 @@ static void print_header(void)
 		cgit_summary_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
 		if (ctx.env.authenticated) {
 			html("</td><td class='form'>");
-			html("<form method='get' action=''>\n");
+			html("<form method='get'>\n");
 			cgit_add_hidden_formfields(0, 1, ctx.qry.page);
 			html("<select name='h' onchange='this.form.submit();'>\n");
 			for_each_branch_ref(print_branch_option, ctx.qry.head);
 			if (ctx.repo->enable_remote_branches)
 				for_each_remote_ref(print_branch_option, ctx.qry.head);
 			html("</select> ");
-			html("<input type='submit' name='' value='switch'/>");
+			html("<input type='submit' value='switch'/>");
 			html("</form>");
 		}
 	} else
@@ -1009,6 +993,11 @@ void cgit_print_pageheader(void)
 		if (ctx.repo->max_stats)
 			cgit_stats_link("stats", NULL, hc("stats"),
 					ctx.qry.head, ctx.qry.vpath);
+		if (ctx.repo->homepage) {
+			html("<a href='");
+			html_attr(ctx.repo->homepage);
+			html("'>homepage</a>");
+		}
 		html("</td><td class='form'>");
 		html("<form class='right' method='get' action='");
 		if (ctx.cfg.virtual_root) {
